/* Originally obtained from Phil Church - pchurch@lanl.gov
   muse.anal.c
   Dynamically records the low and high watermarks for heap and stack
   and calculates the maximum heap and stack size for the program.
   Stack limits are obtained from the Stack Pointer (SP) while heap
   limits are calculated by intercepting calls to brk(), mmap(), sbrk()
*/
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <cmplrs/atom.anal.h>

#define node_name_length 8

/* these keep track of the initial, current, and max 'break' value
   for the apps as well as the initial and maximum stack values */
static long min_HP;
static long cur_HP;
static long max_HP;
static long min_SP;
static long max_SP;

static int call_brk=0;
static int call_sbrk=0;
static int call_mmap=0;

/* the handle for the output file */

static FILE *file;

/* The node name string */
static char node[node_name_length];

/* record call to brk().  This call changes the apps 'break' value. */
void BeforeBrk( long newbrk )
{
  call_brk++;
  cur_HP = newbrk;
  if( cur_HP > max_HP ) max_HP = cur_HP;
}
/* record call to sbrk().  This call increases the apps 'break' value. */
void BeforeSbrk( long size )
{
  call_sbrk++;
  cur_HP += size;
  if( cur_HP > max_HP ) max_HP = cur_HP;
}
/* record call to mmap().  This call maps a file to virtual memory */
void BeforeMmap( size_t size )
{
  call_mmap++;
  cur_HP += size;
  if( cur_HP > max_HP ) max_HP = cur_HP;
}
/* recorn min and max stack pointers */
void GetStack( long SP )
{
  if( SP < min_SP ) min_SP = SP;
  /* don't do the following:
     shared calls screw it up plus stack only grows upward
     if( SP > max_SP ) max_SP = SP;
  */
}
/* create the output file and set the initial address for the apps
   'break' value */
void Initialize( long HP, long SP )
{
  size_t n=node_name_length;

  gethostname( node, n );
  file = fopen( "muse.out", "a" );
  if( file == NULL )
    {
      fprintf( stderr, "muse: unable to create file 'muse.out'\n" );
      exit( 1 );
    }
  min_HP = HP;
  max_HP = HP;
  cur_HP = HP;
  min_SP = SP;
  max_SP = SP;
}
/* print the summary information about this apps heap and stack usage */
void PrintResults()
{
  printf( "\n  Low/High Stack: 0x%9lx 0x%9lx Max Stack Size: %10ld Bytes (%s)\n",
          min_SP, max_SP, max_SP-min_SP, node );
  printf( "\n  Low/High Heap:  0x%9lx 0x%9lx Max Heap  Size: %10ld Bytes (%s)\n",
          min_HP, max_HP, max_HP-min_HP, node );
  printf( "\n  Calls: brk()=%d sbrk()=%d mmap=%d\n",
          call_brk, call_sbrk, call_mmap );
  fprintf( file, "\n  Low/High Stack: 0x%9lx 0x%9lx Max Stack Size: %10ld Bytes (%s)\n",
           min_SP, max_SP, max_SP-min_SP, node );
  fprintf( file, "\n  Low/High Heap:  0x%9lx 0x%9lx Max Heap  Size: %10ld Bytes (%s)\n",
           min_HP, max_HP, max_HP-min_HP, node );
  fprintf( file, "\n  Calls: brk()=%d sbrk()=%d mmap=%d\n",
           call_brk, call_sbrk, call_mmap );
  fclose( file );
}
