#define PROCMON_EXTERN
#define PROCMON_EXTERN_OFF
#include "procmon_info.h"
#include "procmon_rld.h"
/* ========================================================================

   Name
   ====
   procmon_rld_init

   Purpose
   =======
   Initialize data before sampling starts

   Arguments
   =========
   ierr          Intent: out
                 C type: (na) error return value
                 error return value

   Return Values
   =============
   0 if ok

   Program Flow
   ============
   1) process environment variables and set variables

   ======================================================================== */
int procmon_rld_init(
                      void
                      )
{
  int
    ierr=0; /* error return value */
  long long
    part_sec, /* part of a second (some units like microseconds) */
    secs; /* number of seconds */
  /* --------------- */
  /* set some things */
  /* --------------- */
  procmon_s.chain_signal = 0; /* do not chain signal handlers */
  /* other pairs
     procmon_s.which_itimer    = ITIMER_PROF;
     procmon_s.which_sigaction = SIGPROF;
     procmon_s.which_itimer    = ITIMER_VIRTUAL;
     procmon_s.which_sigaction = SIGVTALRM;
  */
  procmon_s.which_itimer = ITIMER_REAL;
  procmon_s.which_sigaction = SIGALRM;
  procmon_info_settings_get( 0, NULL, &(procmon_s.psettings) );
  /* ----------- */
  /* set actions */
  /* ----------- */
  memset(&(procmon_s.action_old),0x00,sizeof(struct sigaction));
  memset(&(procmon_s.action),0x00,sizeof(struct sigaction));
  procmon_s.action.sa_flags = SA_RESTART|SA_SIGINFO;
  procmon_s.action.sa_sigaction = procmon_rld_handler;
  memset(&(procmon_s.action_ignore),0x00,sizeof(struct sigaction));
  procmon_s.action_ignore.sa_handler = SIG_IGN;
  /* ---------------- */
  /* set itimer value */
  /* ---------------- */
  part_sec = (procmon_s.psettings.time%1000)*1000;
  secs = procmon_s.psettings.time/1000;
  procmon_s.value.it_interval.tv_sec = secs;
  procmon_s.value.it_interval.tv_usec = part_sec;
  procmon_s.value.it_value.tv_sec = secs;
  procmon_s.value.it_value.tv_usec = part_sec;
  /* ------------------- */
  /* set output filename */
  /* ------------------- */
  /* ---------------------------- */
  /* open file and print a header */
  /* ---------------------------- */
  /* ---------------------------------------- */
  /* no need to mkstemp...just overwrite file */
  /* ---------------------------------------- */
  /*
    procmon_s.filename_fd = mkstemp( procmon_s.filename );
    if( procmon_s.filename_fd == -1 )
    abort();
    if( close( procmon_s.filename_fd ) != 0 )
    abort();
  */
  if( strlen(procmon_s.psettings.outfile_name) > 0 )
    {
      procmon_s.filename_fp = fopen( procmon_s.psettings.outfile_name, "a" );
    }
  else
    {
      procmon_s.filename_fp = stdout;
    }
  if( procmon_s.filename_fp != NULL )
    {
      procmon_info_settings_print( &(procmon_s.psettings),
                                   procmon_s.filename_fp );
    }
  procmon_s.initialized = !0;
  return( ierr );
}
/* ========================================================================

   Name
   ====
   procmon_rld_terminate

   Purpose
   =======
   free stuff gotten in init call

   Arguments
   =========
   ierr          Intent: out
                 C type: (na) error return value
                 error return value

   Return Values
   =============
   0 if ok

   Program Flow
   ============
   1) process environment variables and set variables

   ======================================================================== */
int procmon_rld_terminate(
                          void
                          )
{
  int
    ierr=0; /* error return value */
  if( procmon_s.initialized == 0 )
    return( ierr );
  procmon_s.initialized = 0;
  if( procmon_s.filename_fp != NULL )
    {
      if( procmon_s.filename_fp != stdout )
        {
          ierr = fclose( procmon_s.filename_fp );
          assert( !ierr );
          /* --------------------------------- */
          /* delete file if not enough samples */
          /* --------------------------------- */
          if( procmon_s.pinfo.num_samples < procmon_s.psettings.min )
            {
              ierr = remove( procmon_s.psettings.outfile_name );
              assert( !ierr );
            }
        }
      procmon_s.filename_fp = NULL;
    }
  return( ierr );
}
/* ========================================================================

   Name
   ====
   procmon_rld_handler

   Purpose
   =======
   Get/print process info info

   Arguments
   =========
   sig           Intent: in
                 C type: int
                 The signal sent to this process.
                 If called independently, the value should be set to 0
                 Currently not used, but could be in the future.

   info          Intent: in
                 C type: siginfo_t *
                 One of the required args for sa_handler

   context       Intent: in
                 C type: void *
                 One of the required args for sa_handler

   Return Values
   =============
   none

   Program Flow
   ============
   1) get info
   2) print info

   ======================================================================== */
void procmon_rld_handler(
                         int sig,
                         siginfo_t *info,
                         void *context
                         )
{
  int procmon_error=0; /* error return value for procmon calls */
  /* -------------------- */
  /* return if in handler */
  /* -------------------- */
  if( procmon_s.in_handler != 0 )
    return;
  procmon_s.in_handler = 1;
  /* ------------------------------------------------------------ */
  /* set to ignore signal (do not want to get signals backing up) */
  /* ------------------------------------------------------------ */
  if( sig != 0 )
    {
      if( sigaction(procmon_s.which_sigaction,
                    &(procmon_s.action_ignore), NULL ) != 0 )
          abort();
    }
  /* -------------- */
  /* get pinfo data */
  /* -------------- */
  procmon_error = procmon_info_get( 0, &(procmon_s.pinfo) );
  /* ------------------------ */
  /* print info               */
  /* file exists and no error */
  /*  - correct interval (or) */
  /*  - final output          */
  /* ------------------------ */
  if( ( procmon_s.filename_fp != NULL ) &&
      ( procmon_error == 0 ) && 
      ( ( procmon_s.psettings.interval > 0 &&
          procmon_s.pinfo.num_samples % procmon_s.psettings.interval == 0 ) ||
        ( procmon_s.pinfo.num_samples > 1 && sig == 0 ) ) )
    {
      procmon_info_print( &(procmon_s.pinfo), procmon_s.filename_fp );
    }
  /* --------------------------------------------- */
  /* chain to previous sig catcher if given signal */
  /* probably do not want to do this...but might   */
  /* --------------------------------------------- */
  if( sig != 0 && procmon_s.chain_signal != 0 )
    {
      if( procmon_s.action_old.sa_flags & SA_SIGINFO &&
          procmon_s.action_old.sa_sigaction != NULL )
        {
          procmon_s.action_old.sa_sigaction(sig, info, context);
        }
      else if( procmon_s.action_old.sa_handler != NULL )
        {
          procmon_s.action_old.sa_handler(sig);
        }
    }
  /* --------------------------------------------------- */
  /* DONE: chain to previous sig catcher if given signal */
  /* --------------------------------------------------- */
  /* ----------------------- */
  /* no longer ingore signal */
  /* ----------------------- */
  if( sig != 0 )
    {
      if (sigaction(procmon_s.which_sigaction,
                    &(procmon_s.action), NULL ) != 0 )
        abort();
    }
  procmon_s.in_handler = 0;
}
/* ========================================================================

   Name
   ====
   procmon_rld_ctor

   Purpose
   =======
   constructor at process start

   Arguments
   =========
   void          Intent: in
                 C type: void
                 no arguments or return values

   Return Values
   =============
   none

   Program Flow
   ============
   1) init: procmon_rld_init
   2) get sampling at beginning
   3) set catcher first then start itimer in case timer goes off

   ======================================================================== */
void procmon_rld_ctor(
                      void
                      )
{
  /* --------- */
  /* init info */
  /* --------- */
  if( procmon_rld_init() != 0 )
    abort();
  /* ------------------------- */
  /* get sampling at beginning */
  /* ------------------------- */
  procmon_rld_handler(0,NULL,NULL);
  /* ---------------------------------------------------------- */
  /* set catcher first then start itimer in case timer goes off */
  /* ---------------------------------------------------------- */
  if (sigaction( procmon_s.which_sigaction, &(procmon_s.action),
                &(procmon_s.action_old)) != 0)
    abort();
  if (setitimer(procmon_s.which_itimer, &(procmon_s.value),
                &(procmon_s.value_old)) != 0)
    abort();
}
/* ========================================================================

   Name
   ====
   procmon_rld_dtor

   Purpose
   =======
   destructor at process termination

   Arguments
   =========
   void          Intent: in
                 C type: void
                 no arguments or return values

   Return Values
   =============
   none

   Program Flow
   ============
   1) reset old catcher after itimer in case timer goes off
   2) get sampling at end
   3) free info

   ======================================================================== */
void procmon_rld_dtor(
                      void
                      )
{
  /* ----------------------------------------------------- */
  /* reset old catcher after itimer in case timer goes off */
  /* ----------------------------------------------------- */
  if (setitimer(procmon_s.which_itimer, &(procmon_s.value_old),
                &(procmon_s.value)) != 0)
    abort();
  if (sigaction(procmon_s.which_sigaction, &(procmon_s.action_old),
                &(procmon_s.action)) != 0)
    abort();
  /* ------------------- */
  /* get sampling at end */
  /* ------------------- */
  procmon_rld_handler(0,NULL,NULL);
  /* --------- */
  /* free info */
  /* --------- */
  if( procmon_rld_terminate() != 0 )
    abort();
}
